@@ -16,9 +16,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
+
+#include "config-ver.h" /* BUILD_VERSION */
 
 #include <wacom-properties.h>
 #include <wacom-util.h>
@@ -28,6 +28,7 @@
 #include <limits.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <stdbool.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <getopt.h>
@@ -513,7 +514,7 @@ static struct deprecated
 	{"GetTabletID", "TabletID"},
 	{"DebugLevel",	"ToolDebugLevel"},
 	{"CommonDBG",	"TabletDebugLevel"},
-	{"GetTabletID",	"TabletID"},
+	{"Serial",	"BindToSerial"},
 	{"PressCurve",	"PressureCurve"},
 	{"TPCButton",	"TabletPCButton"},
 	{"CursorProx",	"CursorProximity"},
@@ -1366,14 +1367,14 @@ static Bool parse_actions(Display *dpy, int argc, char **argv, unsigned long* da
 	int  nwords = 0;
 	char **words = NULL;
 	int n;
-	
+
 	/* translate cmdline commands */
 	words = strjoinsplit(argc, argv, &nwords);
 
 	if (nwords==1 && sscanf(words[0], "%d", &n) == 1)
 	{ /* Mangle "simple" button maps into proper actions */
 		char *nargv[1];
-		
+
 		for (i =  0; i < nwords; i++)
 			free(words[i]);
 		free(words);
@@ -2229,7 +2230,7 @@ static Bool get_mapped_area(Display *dpy, XDevice *dev, int *width, int *height,
 {
 	Atom matrix_prop = XInternAtom(dpy, "Coordinate Transformation Matrix", True);
 	Atom type;
-	int format, i;
+	int format;
 	unsigned long nitems, bytes_after;
 	unsigned long *data;
 	float matrix[9];
@@ -2259,7 +2260,7 @@ static Bool get_mapped_area(Display *dpy, XDevice *dev, int *width, int *height,
 
 	/* XI1 stores 32 bit properties (including float) as long,
 	 * regardless of architecture */
-	for (i = 0; i < ARRAY_SIZE(matrix); i++)
+	for (size_t i = 0; i < ARRAY_SIZE(matrix); i++)
 		matrix[i] = *(float*)(&data[i]);
 
 	TRACE("Current transformation matrix:\n");
@@ -2300,7 +2301,6 @@ static Bool _set_matrix_prop(Display *dpy, XDevice *dev, const float fmatrix[9])
 	unsigned long nitems, bytes_after;
 	float *data;
 	long matrix[9] = {0};
-	int i;
 
 	if (!matrix_prop)
 	{
@@ -2310,7 +2310,7 @@ static Bool _set_matrix_prop(Display *dpy, XDevice *dev, const float fmatrix[9])
 
 	/* XI1 expects 32 bit properties (including float) as long,
 	 * regardless of architecture */
-	for (i = 0; i < ARRAY_SIZE(matrix); i++)
+	for (size_t i = 0; i < ARRAY_SIZE(matrix); i++)
 		*(float*)(matrix + i) = fmatrix[i];
 
 	XGetDeviceProperty(dpy, dev, matrix_prop, 0, 9, False,
@@ -2751,7 +2751,7 @@ out:
 }
 
 
-#ifndef BUILD_TEST
+#ifndef ENABLE_TESTS
 
 #ifdef BUILD_FUZZINTERFACE
 void argsfromstdin(int *argc, char ***argv)
@@ -2795,6 +2795,24 @@ void argsfromstdin(int *argc, char ***argv)
 }
 #endif /* BUILD_FUZZINTERFACE */
 
+static bool check_for_wayland(Display *dpy)
+{
+	bool		has_xwayland_devices = false;
+	XDeviceInfo	*info;
+	int		ndevices, i;
+
+	info = XListInputDevices(dpy, &ndevices);
+	for (i = 0; i < ndevices; i++) {
+		if (strncmp(info[i].name, "xwayland-", 9) == 0) {
+			has_xwayland_devices = true;
+			break;
+		}
+	}
+	XFreeDeviceList(info);
+
+	return has_xwayland_devices;
+}
+
 int main (int argc, char **argv)
 {
 	int c;
@@ -2880,6 +2898,13 @@ int main (int argc, char **argv)
 		return -1;
 	}
 
+	if (check_for_wayland(dpy)) {
+		fprintf(stderr,
+			"Wayland devices found but this tool is incompatible with Wayland. See\n"
+			"https://github.com/linuxwacom/xf86-input-wacom/wiki/Wayland\n");
+		return 1;
+	}
+
 	if (!do_list && !do_get && !do_set)
 	{
 		if (optind < argc)
@@ -2913,15 +2938,16 @@ int main (int argc, char **argv)
 	XCloseDisplay(dpy);
 	return 0;
 }
-#endif
+#endif /* ENABLE_TESTS */
 
-#ifdef BUILD_TEST
+#ifdef ENABLE_TESTS
+#include "wacom-test-suite.h"
 #include <assert.h>
 /**
  * Below are unit-tests to ensure xsetwacom continues to work as expected.
  */
 
-static void test_is_modifier(void)
+TEST_CASE(test_is_modifier)
 {
 	char i;
 	char buff[5];
@@ -2951,7 +2977,7 @@ static void test_is_modifier(void)
 	}
 }
 
-static void test_convert_specialkey(void)
+TEST_CASE(test_convert_specialkey)
 {
 	char i;
 	const char *converted;
@@ -2995,7 +3021,7 @@ static void test_convert_specialkey(void)
 	}
 }
 
-static void test_parameter_number(void)
+TEST_CASE(test_parameter_number)
 {
 	/* If either of those two fails, a parameter was added or removed.
 	 * This test simply exists so that we remember to properly
@@ -3035,7 +3061,7 @@ static void _test_conversion(const param_t *param, const char **words,
 	}
 }
 
-static void test_convert_value_from_user(void)
+TEST_CASE(test_convert_value_from_user)
 {
 	param_t test_nonbool =
 	{
@@ -3081,12 +3107,11 @@ static void test_convert_value_from_user(void)
 }
 
 
+extern void wcm_run_tests(void); /* see wacom-test-suite.c */
+
 int main(int argc, char** argv)
 {
-	test_parameter_number();
-	test_is_modifier();
-	test_convert_specialkey();
-	test_convert_value_from_user();
+	wcm_run_tests();
 	return 0;
 }
 
